package bin.changeName;

import java.util.Vector;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;

public class AdaptFileNames{

	static String CHANGES_FILE = "changes.txt";
	static String OUT_SUFFIX = "_changesLog.txt";

	static boolean gen_output = true;
	
	static Vector path = new Vector();
	
	static BufferedWriter out;
	static Vector oldChanges;
	static Vector changes;
	public static void main(String[] args){
		System.out.println("");
		System.out.println("Adjusting File System Names...");
		System.out.println("");
		if(args.length >0){
			try{
				String topMost = "";
				for(int i=0; i<args.length; i++){
					
					if(args[i].equals("-c")){
						i++;
						CHANGES_FILE = args[i];
					}else if(args[i].equals("..")){
						i++;
						
						if(args[i].charAt(1)==':'){ //if the file path was already absolute with files home
							topMost = args[i];
						}else{ //if the file path was relative without files home 
							topMost += args[i];
						}
					}else{
						if(i>0){
							topMost += " ";
						}
						topMost += args[i];
					}
				}
				
			adaptFileSystem(topMost, CHANGES_FILE);	

			}catch(IOException e){
				e.printStackTrace(new PrintStream(System.out));
			}
			
		}else{
			System.out.println("Please specify a file or folder system");
		}
	}
	
	public static void adaptFileSystem(String topMost, String changesFile) throws IOException{
		BufferedReader in = new BufferedReader(new FileReader(changesFile));
		changes = buildChanges(in);				
		in.close();
		
		//System.out.println("current: "+topMost);
		
		File current = new File(topMost);
		String name = current.getName();
		File output = new File(topMost+AdaptFileNames.OUT_SUFFIX);
		
		
		if(output.exists()){
			BufferedReader old = new BufferedReader(new FileReader(output));
			oldChanges = new Vector();
			String line = old.readLine();
			while(line!=null){
				oldChanges.add(line);
				line = old.readLine();
			}
		}
		
		
		out = new BufferedWriter(new FileWriter(new File(topMost+OUT_SUFFIX)));
		if(oldChanges != null){
			for(int i=0; i<oldChanges.size(); i++){
				out.write((String) oldChanges.elementAt(i));
				out.newLine();
				out.flush();
			}
		}
		topMost = topMost.substring(0, topMost.length() - name.length());
		if(current.isDirectory()){
			System.out.println("Dir: "+topMost+current.getName());
			path.add(topMost.substring(0,topMost.length()-1));
			path.add(current.getName());
			File[] subLevel = current.listFiles();
			for(int i=0; i<subLevel.length; i++){
				traverseFileTree(subLevel[i]);
			}
		}else{
			System.out.println("File: "+topMost+current.getName());
			path.add(topMost.substring(0,topMost.length()-1));
			traverseFileTree(current);
		}
		
		
	}
	
	public static void stopOutput(){
		gen_output = false;
	}
	
	public static void generateOutput(){
		gen_output = true;
	}
	
	public static String changeString(String str) throws IOException{
		stopOutput();
		BufferedReader in = new BufferedReader(new FileReader(CHANGES_FILE));
		changes = buildChanges(in);				
		in.close();
		
		return changeName(str);
	}

	/**************************************************
	*  Traverse the file tree changing file names as you go
	**************************************************/
	public static String traverseFileTree(File current) throws IOException{
		
		String name = current.getName();
		if(current.isDirectory()){
			path.add(name);
			File[] contents = current.listFiles();
			for(int i=0; i<contents.length; i++){
				traverseFileTree(contents[i]);
			}
			path.removeElementAt(path.size()-1);
		}
		
		String newName = changeName(name);
		//System.out.println(getPath() +"\\"+ name);
		current.renameTo(new File(getPath() +"\\"+ newName));
		
		return getPath()+"\\"+newName;
		
	}

	/**************************************************
	*  Get the current path
	**************************************************/
	public static String getPath(){
		String currPath = "";
		for(int i=0; i<path.size(); i++){
			if(i>0){
				currPath += "\\";
			}
			currPath += (String) path.get(i);
		}
		return currPath;
	}
	
	/**************************************************
	*  Changes the name of the file
	**************************************************/
	public static String changeName(String name) throws IOException{
		String origName = name;
		String characters = "";
		boolean changed = false;
		for(int i=0; i<changes.size(); i++){
			Mapping map = (Mapping) changes.get(i);
			String from = map.getOriginalName();
			String to = map.getNewName();
			int indexOf = name.indexOf(from);
			while(indexOf >=0){
				//System.out.println("file:"+name+" searchingFor:\""+from+"\"");
				if(characters.length()>=(from.length()+2)){
					String check = characters.substring(characters.length()-(from.length()+2), characters.length());
					if(!check.equals("\""+from+"\"")){
						if(!characters.equals("")){
							characters += ", ";
						}
						characters += "\""+from+"\"";
					}
				}else{
					characters += "\""+from+"\"";
				}
				changed = true;
				int size = from.length();
				name = name.substring(0,indexOf)+to+name.substring(indexOf+size,name.length());
				indexOf = name.indexOf(from);
			}
		}
		//System.out.println(gen_output);
		if(gen_output && changed){
			//System.out.println(getPath());
			out.write("Filename: \""+getPath()+"\\"+origName+"\"");
			out.newLine();
			out.write("\tchanged to: \""+getPath()+"\\"+name+"\"");
			out.newLine();
			out.write("\tchanged these characters: "+characters);
			out.newLine();
			out.newLine();
			out.flush();
		}
		return name;
	}

	/**************************************************
	*  Builds the list of changes to make
	**************************************************/
	public static Vector buildChanges(BufferedReader in) throws IOException{
		Vector toReturn = new Vector();
		in.readLine();
		in.readLine();
		String line = in.readLine();
		while(line!=null){
			Mapping map = getMapping(line);
			if(map != null){
				//System.out.println("added "+map.getOriginalName()+" "+map.getNewName());
				toReturn.add(map);
			}
			line = in.readLine();
		}
		return toReturn;
	}

	/***************************************
	* Get the mapping of what the desired text should to
	****************************************/
	public static Mapping getMapping(String line){
		//Skip to the first set of quotes
		while(line.charAt(0)!='\"' && line.length()>0){
			line = line.substring(1,line.length());
		}
		if(line.length()>0) line = line.substring(1,line.length()); //go past the open quotes
		
		//parse out the original text
		String firstPart = "";	
		while(line.length()>0 && line.charAt(0)!='\"'){
			if(line.length()>1 && line.charAt(0)=='\\' && line.charAt(1)=='\"'){ //Checks for quotes escape sequence
				firstPart += '\"';
				line = line.substring(2, line.length());
			}else{
				firstPart += line.charAt(0);
				line = line.substring(1,line.length());
			}
		}
		
		//go past the close quotes
		if(line.length()>0) line = line.substring(1,line.length());
		
		//Skip to the second set of quotes
		while(line.charAt(0)!='\"' && line.length()>0){
			line = line.substring(1,line.length());
		}
		
		//go past the open quotes
		if(line.length()>0) line = line.substring(1,line.length());
		
		//parse out the "change to" text
		String secondPart = "";
		while(line.length()>0 && line.charAt(0)!='\"'){
			if(line.length()>1 && line.charAt(0)=='\\' && line.charAt(1)=='\"'){ //Checks for quotes escape sequence
				secondPart += '\"';
				line = line.substring(2, line.length());
			}else{
				secondPart += line.charAt(0);
				line = line.substring(1,line.length());
			}
		}
		
		//System.out.println("firstpart:\""+firstPart+"\" secondPart: \""+secondPart+"\"");
		Mapping map = null;
		if(!firstPart.equals("")){
			map = new Mapping(firstPart, secondPart);
		}
		
		return map;

	}
}
