/**************************************************
*	DroidParser.java
* 	Scans the output of the DroidParser
*
* 	Written by Jacob Bartel
************************************************/
package bin.parser;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.TreeSet;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintStream;
import java.io.IOException;
import java.io.File;

public class DroidParser{
	static HashMap fileTypes = new HashMap();
	static ArrayList warnings = new ArrayList();
	static ArrayList mimeTypes = new ArrayList();
	
	public static void main(String[] args){
		try{
			if(args.length < 1){
				System.out.println("A droid output must be specified");
			}else{
				String fileName = "";
				for(int i=0; i<args.length; i++){
					//System.out.println(fileName);
					if(i>0){
						fileName +=" ";
					}
					fileName += args[i];
					if(fileName.charAt(fileName.length()-1)=='\\'){
						i++;
						if(args[i].charAt(1)==':'){
							fileName = args[i];
						}else{
							fileName += args[i];
						}
					}
				}
				processFile(fileName);
				/*File outputFile = new File(args[0].substring(0,args[0].length()-12)+"Warnings.txt");
				System.out.println(outputFile);
				BufferedWriter out = new BufferedWriter(new FileWriter(outputFile));
				out.write("Collection of Warnings from Droid\n");
				//System.out.print(warnings.size());
				for(int i=0; i<warnings.size(); i++){
					Warnings w = (Warnings) warnings.get(i);
					ArrayList list = w.getWarnings();
					if(list.size()>0){
						out.write("\tFileName: \""+w.getFilePath()+"\"\n");
						for(int j=0; j<list.size(); j++){
							out.write("\t\tWarning: \""+(String)list.get(j)+"\"\n");
							out.flush();
						}
						out.write("\n");
						out.flush();
					}
				}
				out.flush();
				out.close();
				*/
			}
		}catch(IOException e){
			e.printStackTrace(new PrintStream(System.out));
		}
	}
	
	// Process the input file
	public static void processFile(String fileName) throws IOException{
		fileName+="_DROIDOut.xml";
		BufferedReader in = new BufferedReader(new FileReader(new File(fileName)));
		String line = in.readLine();
		while(line != null){
			
			if(line.indexOf("<IdentificationFile")>0){
				
				processFileData(line, in);
			}
			line = in.readLine();
		}
	}
	
	private static void processFileData(String line, BufferedReader in) throws IOException{
		String identQuality = line.substring(36, line.length()-3);
		String filePath = "";
		ArrayList[] types = new ArrayList[2];
		types[0] = new ArrayList();
		types[1] = new ArrayList();
		String warning = "";
		Warnings processWarnings = new Warnings();
		while(line!=null && line.indexOf("</IdentificationFile>")<0){
			if(line.indexOf("<FilePath>")>0){
				filePath = line.substring(line.indexOf("<FilePath>")+10, line.length());
				filePath = filePath.substring(0,filePath.indexOf("</FilePath>"));
				processWarnings.setFilePath(filePath);
				//System.out.println(filePath);
			}			
			if(line.indexOf("<FileFormatHit>")>0){
				String typeName = "";
				String mimeType = "";
				while(line!=null && line.indexOf("</FileFormatHit>") <0){
					if(line.indexOf("<Name>")>0){
						typeName = line.substring(line.indexOf("<Name>")+6, line.indexOf("</Name>"));
						//System.out.println("Name: "+typeName);
					}
					if(line.indexOf("<MimeType>")>0){
						mimeType = line.substring(line.indexOf("<MimeType>")+10, line.indexOf("</MimeType>"));
						//System.out.println("Mime: "+mimeType);
					}
						line = in.readLine();
					if(line.indexOf("<IdentificationWarning")>0){
						warning = line.substring(line.indexOf("<IdentificationWarning>")+23, line.indexOf("</IdentificationWarning"));
						//System.out.println(warning);
					}
				}
				types[0].add(typeName);
				types[1].add(mimeType);
				if(identQuality.equals("Not identified")){
					processWarnings.addWarning("DROID unable to identify file");
					//System.out.println(processWarnings.getFilePath()+": "+warning);
					//addFromFileEnding(filePath);
				}else{		
					//addFromMime(filePath, mimeType);
					//if a warning occurs, we need to add the file to both it's mime type and file ending for jhove
					// also we must add it to the list of warnings
					if(warning.equals("Possible file extension mismatch")){
						//addFromFileEnding(filePath);
						processWarnings.addWarning("DROID indicates "+warning);
					}else if(!warning.equals("")){
						//addFromFileEnding(filePath);
						processWarnings.addWarning(warning);
						//System.out.println(processWarnings.getFilePath()+": "+warning);
					//}else if(!getMimeFromEnding(getEnding(filePath)).equals("application/octet-stream") && mimeType.equals("application/octet-stream")){
					//	processWarnings.addWarning("DROID could not identify file type properly");
					}
				}
			}
			
			line = in.readLine();
		}
		fileTypes.put(filePath, types);
		
		
		warnings.add(processWarnings);
	}
	
	private static void addFromMime(String filePath, String mimeType){
		ArrayList possibleTypes = (ArrayList) fileTypes.get(filePath);
		if(possibleTypes == null)
			possibleTypes = new ArrayList();
		possibleTypes.add(mimeType);
		fileTypes.put(filePath, mimeType);
	}
	
	private static void addFromFileEnding(String filePath){
		String ending = getEnding(filePath);
		ending = ending.toLowerCase();
		addFromMime(filePath, getMimeFromEnding(ending));
	}
	
	private static String getEnding(String filePath){
		String ending = "";
		for(int i=filePath.length(); i>=0; i--){
			char c = filePath.charAt(i);
			if(c=='.'){
				break;
			}else{
				ending += c;
			}
		}
		return ending;
	}
	
	private static String getMimeFromEnding(String ending){
		if(ending.equals("pdf")){
			return "application/pdf";
		}else if(ending.equals("xml")){
			return "text/xml";
		}else if(ending.equals("html") || ending.equals("htm")){
			return "text/html";
		}else if(ending.equals("doc")){
			return "application/octet-stream";
		}else if(ending.equals("png")){
			return "application/octet-stream";
		}else if(ending.equals("tif")){
			return "image/tiff";
		}else if(ending.equals("jpeg") || ending.equals("jpg")){
			return "image/jpeg";
		}else if(ending.equals("ppt")){
			return "application/octet-stream";
		}else if(ending.equals("gif")){
			return "image/gif";
		}else{
			return "application/octet-stream";
		}
	}

}