/**************************************************
*	JhoveParser.java
* 	Parses the output of a Jhove Scanner
*
* 	Written by Jacob Bartel
************************************************/
package bin.parser;


//import bin.parser.FileMetaData;
import java.util.ArrayList;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.File;

public class JhoveParser{


	/****************************************************************
	*	Arrays of array lists for containing file metadata and warnings
	*	0 = aiff-hul module results
	*	1 = ascii-hul module results
	*	2 = bytestream module results
	*	3 = gif-hul module results
	*	4 = html-hul module results
	*	5 = jpeg-hul module results
	*	6 = jpeg2000-hul module results
	*	7 = pdf-hul results
	*	8 = tiff-hul results
	*	9 = UTF8-hul results
	*	10 = wave-hul results
	*	11 = xml-hul results
	****************************************************************/
	static ArrayList[] file_Lists = new ArrayList[12];
	static ArrayList[] warnings_Lists = new ArrayList[12];
	
	
	public static void main(String[] args){
		for(int i=0; i<12; i++){
			file_Lists[i] = new ArrayList();
			warnings_Lists[i] = new ArrayList();
			//warnings_Lists[i].add(new Warnings());
		}
		
		try{
			if(args.length <1){
				System.out.println("Error: Top-most folder or file must be specified");
			}else{	
				String topMost = "";
				for(int i=0; i<args.length; i++){
					
					if(args[i].equals("..")){
						i++;
						if(args[i].charAt(1)==':'){  //if the fileName is already absolute
							topMost = args[i];
						}else{
							if(topMost.charAt(topMost.length()-1)!='\\' && args[i].charAt(0)!='\\'){
								topMost += "\\";
							}
							topMost += args[i];  //if the fileName was not absolute
						}
					}else{
						if(i>0){
						topMost +=" ";
						}
						topMost += args[i];
					}
				}
				//parseJhoveOutput(fileName);
				//parseSetOfJhoveOutput(fileName);
				
			}
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public static void parseSetOfJhoveOutput(String fileName) throws IOException{
		for(int i=0; i<12; i++){
			file_Lists[i] = new ArrayList();
			warnings_Lists[i] = new ArrayList();
			//warnings_Lists[i].add(new Warnings());
		}
		
		parseJhoveOutput(fileName+"_Jhove_aiffOut.xml");
		parseJhoveOutput(fileName+"_Jhove_asciiOut.xml");
		parseJhoveOutput(fileName+"_Jhove_bytestreamOut.xml");
		parseJhoveOutput(fileName+"_Jhove_gifOut.xml");
		parseJhoveOutput(fileName+"_Jhove_htmlOut.xml");
		parseJhoveOutput(fileName+"_Jhove_jpegOut.xml");
		parseJhoveOutput(fileName+"_Jhove_jpeg2000Out.xml");
		parseJhoveOutput(fileName+"_Jhove_pdfOut.xml");
		parseJhoveOutput(fileName+"_Jhove_tiffOut.xml");
		parseJhoveOutput(fileName+"_Jhove_utf8Out.xml");
		parseJhoveOutput(fileName+"_Jhove_waveOut.xml");
		parseJhoveOutput(fileName+"_Jhove_xmlOut.xml");
	}
	
	
	/****************************************************
	*  Begin to  parse through a specific file outputted by Jhove
	****************************************************/
	public static void parseJhoveOutput(String jhoveOutputFile) throws IOException {
		//System.out.println(jhoveOutputFile);
		BufferedReader in = new BufferedReader(new FileReader(new File(jhoveOutputFile)));
		String line = in.readLine();
				
				while(line!=null){
					if(line.indexOf("<repInfo")>=0)  //If we have found the beginning of a file metadata set
						processFileOutput(line, in);
					line = in.readLine();
				}
	}
	
	
	/****************************************************
	* Process through the metadata for a specific file
	*****************************************************/
	public static void processFileOutput(String line, BufferedReader in) throws IOException{
		
		ArrayList files = new ArrayList();
		ArrayList warnings = new ArrayList();
		
		String fileName = line.substring(line.indexOf("\"")+1,line.length()-2);
		String status ="", module="", mimeType="";
		fileName = replaceXMLMarks(fileName);
		//System.out.println(fileName);
		while(line.indexOf("</repInfo>")<0){
			if(line.indexOf("<status>")>=0){
				status = line.substring(line.indexOf("<status>")+8, line.indexOf("</status>"));
				//System.out.println("\t"+status);
			}
			if(line.indexOf("<reportingModule ")>=0){
				module = line.substring(line.indexOf(">")+1, line.length());
				module = module.substring(0, module.indexOf("</"));
				//System.out.println("\t"+module);
			}
			if(line.indexOf("<mimeType>")>=0){
				mimeType = line.substring(line.indexOf("<mimeType>")+10,line.indexOf("</mimeType>"));
				//System.out.println("\t"+mimeType);
			}
			line = in.readLine();
		}
		ArrayList warningsList = getWarningsList(module);
		if(!status.equals("Well-Formed and valid")){
			String warning;
			//System.out.println(warningsList.size());
			if(status.equals("Well-Formed, but not valid")){
				warning = "Jhove indicates the file is well formed, but not valid";
				//System.out.println(warning);
				addWarning(warningsList, fileName, warning);
			}else if(status.equals("Well-Formed")){
				warning = "Jhove indicates the file is well formed, but could not check for validity";
				//System.out.println(warning);
				addWarning(warningsList, fileName, warning);
			}else if (status.equals("Not well-formed")){
				warning = "Jhove indicates status of Not well-formed";
				addWarning(warningsList, fileName, warning);
			}else{
				warning = "Jhove indicates status of "+status;
				//System.out.println(fileName);
				//System.out.println(warning);
				addWarning(warningsList, fileName, warning);
			}
			//System.out.println(warningsList.size());
		}else{ 
			String warning = "Jhove indicates file possibly of type "+mimeType;
			addWarning(warningsList, fileName, warning);
		}
	}
	
	/**************************************
	* Get the warnings list associated with a specific module
	***************************************/
	public static ArrayList getWarningsList(String module){
		module = module.toLowerCase();
		//System.out.println("'"+module+"'");
		
		if(module.equals("aiff-hull")){
			//System.out.println(0);
			return warnings_Lists[0];
		}else if(module.equals("ascii-hul")){
			//System.out.println(1);
			return warnings_Lists[1];
		}else if(module.equals("gif-hul")){
			//System.out.println(3);
			return warnings_Lists[3];
		}else if(module.equals("html-hul")){
			//System.out.println(4);
			return warnings_Lists[4];
		}else if(module.equals("jpeg-hul")){
			//System.out.println(5);
			return warnings_Lists[5];
		}else if(module.equals("jpeg2000-hul")){
			//System.out.println(6);
			return warnings_Lists[6];
		}else if(module.equals("pdf-hul")){
			//System.out.println(7);
			return warnings_Lists[7];
		}else if(module.equals("tiff-hul")){
			//System.out.println(8);
			return warnings_Lists[8];
		}else if(module.equals("utf8-hul")){
			//System.out.println(9);
			return warnings_Lists[9];
		}else if(module.equals("wave-hul")){
			//System.out.println(10);
			return warnings_Lists[10];
		}else if(module.equals("xml-hul")){
			//System.out.println(11);
			return warnings_Lists[11];
		}else{
			//System.out.println(2);
			return warnings_Lists[2];  //By default, returns the bytestream module
		}
	}
	
	/******************************************
	* Add a Warning to the set of warnings for that file
	* If no set exists, it will create a new set of warnings
	*******************************************/
	public static void addWarning(ArrayList listOfWarnings, String fileName, String warning){
		
		for(int i=0; i<listOfWarnings.size(); i++){
			Warnings warnings = (Warnings) listOfWarnings.get(i);
			if(warnings.getFilePath().equals(fileName)){
				warnings.addWarning(warning);
				return;
			}
		}
		Warnings warnings = new Warnings();
		warnings.setFilePath(fileName);
		warnings.addWarning(warning);
		listOfWarnings.add(warnings);
	}
	
	/*********************************************
	* Replace XML marks such as %20 or %5C with the appropriate
	* ASCII symbol(s)
	*********************************************/
	public static String replaceXMLMarks(String fileName){
		int slashPos = fileName.indexOf("%5C");
		while(slashPos >=0){
			fileName = fileName.substring(0,slashPos)+"\\"+fileName.substring(slashPos+3,fileName.length());
			slashPos = fileName.indexOf("%5C");
		}
		int spacePos = fileName.indexOf("%20");
		while(spacePos >=0){
			fileName = fileName.substring(0,spacePos)+" "+fileName.substring(spacePos+3,fileName.length());
			spacePos = fileName.indexOf("%20");
		}
		return fileName;
	}

}

