/**************************************************
*	PrintFileTypes.java
* 	Prints file types as read in by DroidParser and JhoveParser
*
*	The output file is name <filename>_FileTypes.txt
*
* 	Written by Jacob Bartel
************************************************/
package bin.parser;

import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.File;

public class PrintFileTypes{
	static HashMap DroidFileTypes = DroidParser.fileTypes;
	static ArrayList[] JhoveWarningsList = JhoveParser.warnings_Lists;	

	public static void main(String[] args){
		try{
			if(args.length<1){
				System.out.println("Must specifiy to most file or directory to print warnings from");
			}else{
			
			
				//Parse for Droid and Jhove
				DroidParser.main(args);
				JhoveParser.main(args);
				
				
				//All the arguments are compiled into one file name
				String fileName = "";
				for(int i=0; i<args.length; i++){
					if(i>0){
						fileName+=" ";
					}
					fileName += args[i];
					//if the files home was specified
					if(fileName.charAt(fileName.length()-1)=='\\'){
						i++;
						
						if(args[i].charAt(1)==':'){ //if the file path was already absolute with files home
							fileName = args[i];
						}else{ //if the file path was relative without files home 
							fileName += args[i];
						}
					}
				}
				
				setUpFilesAndPrint(fileName);
			}
		
		}catch(IOException e){
			System.out.println(e);
		}
	}
	
	public static void setUpFilesAndPrint(String fileName) throws IOException{
		fileName += "_FileTypes.txt";
		//System.out.println(fileName);
		File outFile = new File(fileName);
		if(!outFile.exists()){
			outFile.createNewFile();
		}
		BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
		out.write("FileTypes from Droid and Jhove");
		out.newLine();
		printFileTypes(out);
		out.close();
	}
	
	public static void printFileTypes(BufferedWriter out) throws IOException{
		ArrayList fileNames = new ArrayList();
		Iterator iterator = DroidFileTypes.keySet().iterator();
		while(iterator.hasNext()){
			fileNames.add(iterator.next());
		}
		
		for(int i=0; i<fileNames.size(); i++){
			String fileName = (String) fileNames.get(i);
			//System.out.println(fileName);

			ArrayList[] droidTypes = (ArrayList[]) DroidParser.fileTypes.get(fileName);
			ArrayList pastTypes = new ArrayList();
			ArrayList droidFileTypes = new ArrayList();
			for(int n=0; n<droidTypes[0].size(); n++){
				String type = "Droid indicates file possibly of type "+(String)droidTypes[0].get(n);
				String mimeType = (String) droidTypes[1].get(n);
				if(!mimeType.equals("")){
					type += " with MIME "+mimeType;
				}
				boolean found = false;
				for(int pt=0; pt<pastTypes.size(); pt++){
					String message = (String) pastTypes.get(pt);
					if(message.equals(type)){
						found = true;
						break;
					}
				}
				if(!found){
					droidFileTypes.add(type);
				}
			}

			ArrayList jhoveFileTypes = new ArrayList();

			
			for(int n=0; n<JhoveWarningsList.length; n++){
				for(int k=0; k<JhoveWarningsList[n].size();k++){		
					Warnings warnings = (Warnings) JhoveWarningsList[n].get(k);
					ArrayList list = warnings.getWarnings();
					if(fileName.equals(warnings.getFilePath()) && list.size()>0){
						for(int l=0; l<list.size(); l++){
							String warning = (String) list.get(l);
							if(warning.indexOf("Jhove indicates file possibly of type ")>=0){
								//System.out.println("\t"+warning.substring(38, warning.length()));
								jhoveFileTypes.add(warning.substring(38, warning.length()));
							}
						}
					}
				}
			}
			out.newLine();
			out.write("\tFilename: \""+fileName+"\"");
			out.newLine();
			out.flush();
			if(!(droidFileTypes.size()==0 && jhoveFileTypes.size()==0)){
				if(droidFileTypes.size()>0){
					out.write("\t\tDroid lists possible types as: ");
					out.newLine();
					for(int j=0; j<droidFileTypes.size(); j++){
						String type = (String)droidFileTypes.get(j);
						out.write("\t\t\t"+type.substring(38,type.length()));
						out.newLine();
					}
					out.flush();
				}
				if(jhoveFileTypes.size()>0){
					out.write("\t\tJhove lists possible types as:");
					out.newLine();
					for(int j=0; j<jhoveFileTypes.size(); j++){
						out.write("\t\t\t"+(String)jhoveFileTypes.get(j));
						out.newLine();
					}
					out.flush();
				}
			}else{
				out.write("\t\tFile type could not be identified");
				out.newLine();
				out.flush();
			}
		}
	}
}