/**************************************************
*	PrintWarnings.java
* 	Prints warnings as read in by DroidParser and JhoveParser
*
*	The output file is name <filename>_Warnings.txt
*
* 	Written by Jacob Bartel
************************************************/
package bin.parser;

import java.util.ArrayList;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.io.File;

public class PrintWarnings{
	static ArrayList DroidWarningsList = DroidParser.warnings;
	static ArrayList[] JhoveWarningsList = JhoveParser.warnings_Lists;

	public static void main(String[] args){
		try{
			if(args.length<1){
				System.out.println("Must specifiy to most file or directory to print warnings from");
			}else{
			
				//Parse for Droid and Jhove
				DroidParser.main(args);
				JhoveParser.main(args);
				
				
				//All the arguments are compiled into one file name
				String fileName = "";
				for(int i=0; i<args.length; i++){
					if(i>0){
						fileName+=" ";
					}
					fileName += args[i];
					//if the files home was specified
					if(fileName.charAt(fileName.length()-1)=='\\'){
						i++;
						
						if(args[i].charAt(1)==':'){ //if the file path was already absolute with files home
							fileName = args[i];
						}else{ //if the file path was relative without files home 
							fileName += args[i];
						}
					}
				}
				
				setUpFilesAndPrint(fileName);
			}
		
		}catch(Exception e){
			e.printStackTrace(new PrintStream(System.out));
		}
	}

	public static void setUpFilesAndPrint(String fileName) throws IOException{
		fileName+="_Warnings.txt";
		File outFile = new File(fileName);
		if(!outFile.exists()){
			//System.out.println("create");
			outFile.createNewFile();
		}
		BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
		out.write("Collection of Warnings from Droid and Jhove");
		out.newLine();
		printWarnings(out);
		out.close();
	}
	
	public static void printWarnings(BufferedWriter out) throws IOException{
		for(int i=0; i<DroidWarningsList.size(); i++){
			boolean printed = false;
			Warnings droidWarnings = (Warnings) DroidWarningsList.get(i);
			String fileName = droidWarnings.getFilePath();
			ArrayList list = droidWarnings.getWarnings();
			if(list.size()>0){
				out.newLine();
				out.write("\tFileName: \""+fileName+"\"");
				out.newLine();
				printed = true;
			}
			boolean fileMismatch = false;
			
			//Print all the warnings from Droid
			for(int j=0; j<list.size(); j++){
				String warning = (String) list.get(j);
				out.write("\t\tWarning: \""+warning+"\"");
				out.newLine();
				out.flush();
				
				//If the warning is a possible file extension mismatch, print all possible file types
				if(warning.indexOf("Possible file extension mismatch")>=0){
					ArrayList[] droidTypes = (ArrayList[]) DroidParser.fileTypes.get(fileName);
					ArrayList pastTypes = new ArrayList();
					for(int n=0; n<droidTypes[0].size(); n++){
						String type = "Droid indicates file possibly of type "+(String)droidTypes[0].get(n);
						String mimeType = (String) droidTypes[1].get(n);
						if(!mimeType.equals("")){
							type += " with MIME "+mimeType;
						}
						boolean found = false;
						for(int pt=0; pt<pastTypes.size(); pt++){
							String message = (String) pastTypes.get(pt);
							if(message.equals(type)){
								found = true;
								break;
							}
						}
						if(!found){
							pastTypes.add(type);
							out.write("\t\t\t"+type);
							out.newLine();
							out.flush();
						}
					}
					
					for(int n=0; n<JhoveWarningsList.length; n++){
						for(int k=0; k<JhoveWarningsList[n].size();k++){
							Warnings warnings = (Warnings) JhoveWarningsList[n].get(k);
							list = warnings.getWarnings();
							if(fileName.equals(warnings.getFilePath())){
								for(int l=0; l<list.size(); l++){
									warning = (String) list.get(l);
									//System.out.println("\t"+warning);
									if(warning.indexOf("Jhove indicates file possibly of type ")>=0){
										boolean found = false;
										for(int pt=0; pt<pastTypes.size(); pt++){
											String message = (String) pastTypes.get(pt);
											if(message.equals(warning)){
												found = true;
												break;
											}
										}
										if(!found){
											pastTypes.add(warning);
											out.write("\t\t\t"+warning);
											out.newLine();
											out.flush();
										}
									}
								}
							}
						}
					}
					
				}
			}
			
			ArrayList pastTypes = new ArrayList();
			boolean[] listNotValid = new boolean[JhoveWarningsList.length];
			
			//Print all the warnings from Jhove
			for(int n=0; n<JhoveWarningsList.length; n++){
				for(int k=0; k<JhoveWarningsList[n].size();k++){
					
					Warnings warnings = (Warnings) JhoveWarningsList[n].get(k);
					list = warnings.getWarnings();
					
					if(fileName.equals(warnings.getFilePath()) && list.size()>0){
						boolean validForList = true;
						ArrayList toPrint = new ArrayList();
						
						for(int l=0; l<list.size(); l++){
							String warning = (String) list.get(l);
							if(warning.equals("Jhove indicates status of Not well-formed")){
								listNotValid[n] = true;
							}
							
							if(warning.indexOf("Jhove indicates file possibly of type ") < 0 && !warning.equals("Jhove indicates status of Not well-formed")){
								boolean found = false;
								for(int pt=0; pt<pastTypes.size(); pt++){
									String message = (String) pastTypes.get(pt);
									if(message.equals(warning)){
										found = true;
										break;
									}
								}
								if(!found){
									pastTypes.add(warning);
									toPrint.add("\t\tWarning: "+warning);
								}
							}
						}
						if(!printed && toPrint.size()>0){
							out.newLine();
							out.write("\tFileName: \""+fileName+"\"");
							out.newLine();
							printed = true;
							
							for(int tp=0; tp<toPrint.size(); tp++){
								out.write((String)toPrint.get(tp));
								out.newLine();
								out.flush();
							}
						}
						
						break;
					}
				}
			}
			boolean foundValidOverall = false;
			for(int count=0; count<listNotValid.length; count++){
				if(listNotValid[count]==false){
					foundValidOverall = true;
					break;
				}
			}
			if(!foundValidOverall){
				if(!printed){
					out.newLine();
					out.write("\tFilename: \""+fileName+"\"");
					out.newLine();
				}
				out.write("\t\tWarning: Jhove found that file is not well-formed for any type");
				out.newLine();
				out.flush();
				String droidMime =(String) DroidParser.mimeTypes.get(i);
				if(!droidMime.equals("")){
					out.write("\t\t\tDroid identified file as type "+droidMime);
					out.newLine();
				}else{
					out.write("\t\t\tDroid unable to identify file as well-formed");
					out.newLine();
				}
				out.flush();
			}
		}
	}
}